#include <iostream>
#include <ctime>
#include <iomanip>
#include <algorithm>
#include <cstdlib>
#include <time.h>
#include <fstream>
#include <string>
#include <termios.h>
#include <unistd.h>
#include <stdio.h>
#include <chrono>
#include <thread>

// This game will have the character falling down a massive hole
// And avoiding obstacles that are randomly generated
// Each "level" is a layer of rock that the person is diving down
// And each level is an array within the array that is "map"
// So in the array "map" there is "height" levels that are only
// Generated when the player is within render distance (screen_height)
// Of that level, this continues until the player hits an object.


// Variables

int distance_from_right = 10; // How far accross the character is (from the right)
const int width = 20; // Maximum width of the hole
int screen_height = 35; // Render distance that the levels are loaded up to
int turn = 0;
int difficulty = 4; // higher is easier, 1 is hardest (impossible as it obstacles have a 1 in  1 chance of spawning)
const int height = 9999;
int left_wall_position = 0;
int right_wall_position = width;

bool alive = true; //Will become false when you die :(

std::string map[height][width];
std::string mode = "border"; // Can be "loop" or "border" mode
std::string character_icon = "☹";


// Functions

std::string upper(std::string& text) { // Converts string to uppercase
    //std::string uppertext;
    for (auto& c : text) {
        c = toupper(c);
    }

    return text;
}


int print_debug_info() { // Prints variables
  std::cout << "Debug information:" << "\n";
  std::cout << "turn " << turn << "\n";
  std::cout << "left_wall_position " << left_wall_position << "\n";
  std::cout << "right_wall_position " << right_wall_position << "\n";
  std::cout << "difficulty " << difficulty << "\n";
  std::cout << "screen_height " << screen_height << "\n";
  std::cout << "width " << width << " CANNOT MODIFY\n";
  std::cout << "height " << height << " CANNOT MODIFY\n";
  std::cout << "distance_from_right " << distance_from_right << "\n";
  std::cout << "character_icon " << character_icon << "\n";
  std::cout << "mode " << mode << "\n";
  std::cout << "alive " << alive << "\n";
  return 0;
}

int getch(void)
{
    struct termios oldattr, newattr;
    int ch;
    tcgetattr( STDIN_FILENO, &oldattr );
    newattr = oldattr;
    newattr.c_lflag &= ~( ICANON | ECHO );
    tcsetattr( STDIN_FILENO, TCSANOW, &newattr );
    ch = getchar();
    tcsetattr( STDIN_FILENO, TCSANOW, &oldattr );
    return ch;
}


int random_number(int min, int max) { // Generates a random number between min and max
  int randNum = rand()%(max-min + 1) + min;
  return randNum;
}

int clear() {
   std::cout << "\033[2J\033[1;1H"; // Clear the console screen
   return 0;
}

int create_level(int level) {
  if (mode == "border") {
    // Randomly change position of left wall
    if ((random_number(0,10) <= 2 && left_wall_position <= (width/4)) || left_wall_position == 0) {  // Maybe the wall will go out
      left_wall_position++;
      map[level][left_wall_position] = "\\";
    } else if (random_number(0,10) <= 4 && left_wall_position > 1) { // Or it could go in if there's space
      left_wall_position--;
      map[level][left_wall_position] = "/";
    } else { // It might just stay the same
      map[level][left_wall_position] = "|";
    }
    for (int i = 0; i < left_wall_position; i++) { // Anything that is outside of the wall is replaced with " " so it doesn't look ugly
      map[level][i] = " ";
    }


    // Randomly change position of right wall
    if ((random_number(0,10) <= 2 && right_wall_position > ((width/4)*3)) || right_wall_position == width) { //Maybe the wall will go out
      right_wall_position--;
      map[level][right_wall_position] = "/";
    } else if (random_number(0,10) <= 4 && right_wall_position < width-1) { // Or it could go in if there's space
      right_wall_position++;
      map[level][right_wall_position] = "\\";
    } else { // It might just stay the same
      map[level][right_wall_position] = "|";
    }
    for (int i = width; i > right_wall_position; i--) { // Anything that is outside of the wall is replaced with " " so it doesn't look ugly
      map[level][i] = " ";
    }
  }
  else {
    map[level][0] = "|";
    map[level][width] = "|";
  }
  

  for (int j = 1; j < width; j++) {
    if (map[level][j] == "") {
      if (random_number(1,difficulty) == 1) { // If difficulty is lower (harder) then there's a higher chance of an obstacle
	map[level][j] = "‾"; // Assigns the obstacle character
      }
      else {
	map[level][j] = " "; // Fill the rest of the array with Spaces
      }
    }
  }
  return 0;
}

// Function to print a level to the console.
int print_level(int level) {
  // The Background of the hole you're falling down is orangy brown so the program needs to know wether it's printed the left wall because then it will start making the backdrop orange and once it's passed the right wall (second wall) it stops making the backdrop orange so that's what the booleans do.
  bool left_wall_passed = false; // Boolean that says wether the left wall has been printed yet, used to know whether to print the wallpapaer as orange or no different colour
  bool right_wall_passed = false; // see above, but right
  std::cout << "\e[5;31;104m" << std::setfill('0') << std::setw(5) << level << "\e[m "; // the 5 makes the numbers blink
  for (int j = 0; j <= width; j++) {
    if (map[level][j] == " " && left_wall_passed && ! right_wall_passed) {
      std::cout << "\e[0;30;43m" <<  map[level][j] << "\e[m";
    } else if (map[level][j] == "|" || map[level][j] == "\\" || map[level][j] == "/") {
      std::cout << "\e[1;31;43m" <<  map[level][j] << "\e[m";
      if (! left_wall_passed && ! right_wall_passed) {
	left_wall_passed = true;
      } else {
	right_wall_passed = true;
      }
    } else if (map[level][j] == "‾") {
      std::cout << "\e[1;31;43m" <<  map[level][j] << "\e[m";
    } else if (map[level][j] == character_icon) {
      std::cout << "\e[0;36;43m" <<  map[level][j] << "\e[m";
    } else {
      std::cout << map[level][j];
    }
  }
  std::cout << "\n";
  return 0;
}

// 
int end_turn() {
  turn = turn + 1; // Increment the turn counter

  // Checks if you're dead
  // Border checks
  if (distance_from_right <= 0) {
    alive = false;
  } else if (distance_from_right >= width) {
    alive = false;
  }

  // Check for obstacles
  if (map[turn][distance_from_right] != " ") {
    alive = false;
  }

  // Check if you've won!
  if (turn >= height) {
    std::cout << "You won!\n";
    
  }
  
  // Creates a new level at the end of each turn
  create_level(turn+screen_height);

  // Print the map with the new level
  clear();
  map[turn][distance_from_right] = character_icon;
  for (int i = turn; i < turn + screen_height; i++) {
    print_level(i);
  }
  //std::cout << "lwp" << left_wall_position << "  rwp" << right_wall_position << "\n";
  return 0;
}

int main() {
  clear(); // CLear the console screen
  srand(time(0));   // Use current time as seed for random generator
  
  // Make the 0 level, looks a bit different to the others
  for (int i = 0; i <= width; i++) {
    if (i == 0) {
      map[0][i] = "/";
    } else if (i == width) {
      map[0][i] = "\\";
    } else {
      map[0][i] = "‾";
    }
  }
  map[0][distance_from_right] = character_icon;
  // Generate in initial empty map
  // It loops over each level and assigns things to it until it's reached the screen_height
  for (int i = turn; i <= turn + screen_height; i++) {
    if (i != 0) { // Doesn't make a new level at 0 as that is different and was made in the last for loop
      create_level(i);
    }
    //map[turn][distance_from_right] = character_icon; // Add the player character at the current position
    print_level(i); // Print the current level
  }
  
  while (alive == true && turn < height) {
    std::string direction;
    direction = getch(); // Read user input for direction
    if (direction == "d") { // Right
      distance_from_right++;
      if (distance_from_right == width && mode == "loop") {
	distance_from_right = 1;
      }
      end_turn(); 
    }
    else if (direction == "a") { //LeftOA
      distance_from_right--;
      if (distance_from_right == 0 && mode == "loop") {
	distance_from_right = width-1;
      }
      end_turn();
    }
    else if (direction == "s") { // Straight down
      end_turn();
    }
    else if (direction == "w") { // Print debug information
      print_debug_info();
    }
    else if (direction == "e") { // Modify variables
      std::string actually;
      std::cout << "Actually edit variable? (y/n) " << "\n";
      std::cin >> actually;
      if (upper(actually) == "Y") {
	print_debug_info();
	std::string variable;
	std::cout << "Which variable do you want to modify?" << "\n";
	std::cin >> variable;
	std::string new_value;
	if (upper(variable) == "TURN") {
	  std::cout << "New value: ";
	  std::cin >> new_value;
	  turn = std::stoi(new_value);
	} else if (upper(variable) == "LEFT_WALL_POSITION") {
	  std::cout << "New value: ";
	  std::cin >> new_value;
	  left_wall_position = std::stoi(new_value);
	} else if (upper(variable) == "RIGHT_WALL_POSITION") {
	  std::cout << "New value: ";
	  std::cin >> new_value;
	  right_wall_position = std::stoi(new_value);
	} else if (upper(variable) == "DIFFICULTY") {
	  std::cout << "New value: ";
	  std::cin >> new_value;
	  difficulty = std::stoi(new_value);
	} else if (upper(variable) == "SCREEN_HEIGHT") {
	  std::cout << "New value: ";
	  std::cin >> new_value;
	  screen_height = std::stoi(new_value);
	} else if (upper(variable) == "DISTANCE_FROM_RIGHT") {
	  std::cout << "New value: ";
	  std::cin >> new_value;
	  distance_from_right = std::stoi(new_value);
	} else if (upper(variable) == "CHARACTER_ICON") {
	  std::cout << "New value: ";
	  std::cin >> character_icon;
	} else if (upper(variable) == "MODE") {
	  std::cout << "New value: ";
	  std::cin >> mode;
	} else if (upper(variable) == "ALIVE") {
	  std::cout << "New value: ";
	  std::cin >> new_value;
	  alive = std::stoi(new_value);
	} else {
	  std::cout << "Variable doesn't exist or is a constant value" << "\n";
	}
        for (int i = turn; i <= turn + screen_height; i++) {
	  create_level(i);
	  map[turn][distance_from_right] = character_icon; // Add the player character at the current position
	  print_level(i); // Print the current level
	}
      }
    }
  }

  // Post game stuff
  clear();

  // Getting the time
  std::time_t now = std::chrono::system_clock::to_time_t(std::chrono::system_clock::now());
  std::stringstream ss;
  ss << std::put_time(std::localtime(&now), "%Y-%m-%d %H:%M:%S");
  std::string current_date = ss.str();

  // telling you you lost and what your score is
  // Score is turn - 1 as you will be one turn above the one you got eliminated on
  std::cout << "\nYou died :(\nwomp womp\n";
  std::cout << "You had a score of " << turn - 1 << "\n";
  std::cout << "What is your name for the hiscoreboard?" << "\n";
  std::string name; 
  std::cin >> name; // Asks for player name

  // Commands that write score to hiscore file and rearrange the file into order
  std::string add_score_command = "echo '" + std::to_string(turn - 1) + " - " + name + " - " + current_date + "' >> hiscores.txt";
  std::system(add_score_command.c_str()); // Writes score to file
  std::system("sort -nr -o hiscores.txt hiscores.txt"); // rearranges file
  std::system("echo ''");
  std::system("echo '=== HISCORES ==='");
  std::system("echo ''");
  std::system("cat hiscores.txt");
  return 0;
}

// DONE
// Generates new level (empty level)
// Allows you to fall
// death detection
   // End screen
// random obstacles
// side to side movement
// Keybinds rather than typing inputs
// Walls get narrower and wider
// Hiscores

// TODO
// continuous falling
// Win ? (It's kinda meant to be an infinite game but I added a limit for technical reasons, should I have a win bit if anyone even gets there?)

// KNOWN BUGS
// If screen_height is higher than height then it wont work
// If screen_height is higher than screen you can't see urself
// Seg Faults when reach the height of the map
// When you get in render distance of the end then it crashes, not really important as the end is far away!
